@@ -35,8 +35,6 @@ struct _PangoCairoATSUIFont
   PangoATSUIFont font;
   PangoCairoFontPrivate cf_priv;
 
-  ATSUFontID font_id;
-
   double size;
   int absolute_size;
 };
@@ -52,23 +50,6 @@ static cairo_font_face_t *pango_cairo_atsui_font_create_font_face           (Pan
 static PangoFontMetrics  *pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
                                                                             PangoContext    *context);
 
-
-/**
- * pango_cairo_atsui_font_get_atsu_font_id:
- * @cafont: A #PangoCairoATSUIFont
- *
- * Returns the ATSUFontID of a font.
- *
- * Return value: the ATSUFontID associated to @cafont.
- *
- * Since: 1.12
- */
-ATSUFontID
-pango_cairo_atsui_font_get_atsu_font_id (PangoCairoATSUIFont *cafont)
-{
-  return cafont->font_id;
-}
-
 static void
 cairo_font_iface_init (PangoCairoFontIface *iface)
 {
@@ -97,12 +78,19 @@ pango_cairo_atsui_font_get_glyph_extents (PangoFont      *font,
 					       logical_rect);
 }
 
+/* XXX remove when we require cairo 1.6 */
+#undef cairo_atsui_font_face_create_for_atsu_font_id
+cairo_public cairo_font_face_t *
+cairo_atsui_face_create_for_atsu_font_id (ATSUFontID font_id);
+
 static cairo_font_face_t *
 pango_cairo_atsui_font_create_font_face (PangoCairoFont *font)
 {
-  PangoCairoATSUIFont *cafont = (PangoCairoATSUIFont *) (font);
+  PangoATSUIFont *afont = (PangoATSUIFont *) (font);
+  ATSUFontID font_id;
 
-  return cairo_atsui_font_face_create_for_atsu_font_id (cafont->font_id);
+  font_id = pango_atsui_font_get_atsu_font_id (afont);
+  return cairo_atsui_font_face_create_for_atsu_font_id (font_id);
 }
 
 static int
@@ -134,6 +122,7 @@ pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
                                                    PangoContext   *context)
 {
   PangoCairoATSUIFont *cafont = (PangoCairoATSUIFont *) font;
+  PangoATSUIFont *afont = (PangoATSUIFont *) font;
   ATSFontRef ats_font;
   ATSFontMetrics ats_metrics;
   PangoFontMetrics *metrics;
@@ -143,7 +132,7 @@ pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
   PangoLanguage *language = pango_context_get_language (context);
   const char *sample_str = pango_language_get_sample_string (language);
 
-  ats_font = FMGetATSFontRefFromFont (cafont->font_id);
+  ats_font = FMGetATSFontRefFromFont (pango_atsui_font_get_atsu_font_id (afont));
 
   ATSFontGetHorizontalMetrics (ats_font, kATSOptionFlagsDefault, &ats_metrics);
 
@@ -179,10 +168,9 @@ static PangoFontDescription *
 pango_cairo_atsui_font_describe_absolute (PangoFont *font)
 {
   PangoFontDescription *desc;
-  PangoATSUIFont *afont = (PangoATSUIFont *) font;
   PangoCairoATSUIFont *cafont = (PangoCairoATSUIFont *) font;
 
-  desc = pango_font_description_copy (afont->desc);
+  desc = pango_font_describe (font);
   pango_font_description_set_absolute_size (desc, cafont->absolute_size);
 
   return desc;
@@ -227,6 +215,7 @@ _pango_cairo_atsui_font_new (PangoCairoATSUIFontMap     *cafontmap,
   ATSUFontID font_id;
   double size;
   double dpi;
+  double m;
   cairo_matrix_t font_matrix;
 
   postscript_name = _pango_atsui_face_get_postscript_name (face);
@@ -275,11 +264,11 @@ _pango_cairo_atsui_font_new (PangoCairoATSUIFontMap     *cafontmap,
   cafont = g_object_new (PANGO_TYPE_CAIRO_ATSUI_FONT, NULL);
   afont = PANGO_ATSUI_FONT (cafont);
 
-  afont->desc = pango_font_description_copy (desc);
-  afont->face = face;
+  _pango_atsui_font_set_font_description (afont, desc);
+  _pango_atsui_font_set_face (afont, face);
 
   size = (double) pango_font_description_get_size (desc) / PANGO_SCALE;
-  cafont->font_id = font_id;
+  _pango_atsui_font_set_atsu_font_id (afont, font_id);
 
   if (context)
     {
@@ -298,11 +287,19 @@ _pango_cairo_atsui_font_new (PangoCairoATSUIFontMap     *cafontmap,
 
   cafont->size = size;
 
-  /* If necessary, apply a shear matrix, matching what Cocoa does. */
+  /* When synthesizing italics, apply a shear matrix matching what Cocoa
+   * does. Cairo quartz had transformed text wrong before 1.5.13, stay
+   * backwards compatible until pango requires a new enough cairo.
+   */
+  if (cairo_version () >= CAIRO_VERSION_ENCODE(1,5,13))
+    m = -0.25;
+  else
+    m = 0.25;
+
   if (synthesize_italic)
     cairo_matrix_init (&font_matrix,
                        1, 0, 
-                       0.25, 1,
+                       m, 1,
                        0, 0);
   else
     cairo_matrix_init_identity (&font_matrix);
